@@ -1,7 +1,4 @@
-try:
-    import cStringIO as StringIO
-except ImportError:
-    import StringIO
+from six import BytesIO, text_type
 
 from django.http import HttpResponse
 from django.contrib.contenttypes.models import ContentType
@@ -16,7 +13,7 @@ from collections import namedtuple
 from decimal import Decimal
 from numbers import Number
 
-from model_introspection import (
+from report_utils.model_introspection import (
     get_relation_fields_from_model,
     get_properties_from_model,
     get_direct_fields_from_model,
@@ -27,16 +24,38 @@ DisplayField = namedtuple("DisplayField", "path path_verbose field field_verbose
 
 class DataExportMixin(object):
     def build_sheet(self, data, ws, sheet_name='report', header=None, widths=None):
+        # Try to detect the openpyxl version, since the API changes
+        # significantly from v1 to v2
+        try:
+            from openpyxl import __major__ as openpyxl_major_version
+            old_openpyxl = openpyxl_major_version < 2
+        except ImportError:
+            old_openpyxl = False
+        if old_openpyxl:
+            first_row = 0
+            column_base = 0
+        else:
+            first_row = 1
+            column_base = 1
+
         ws.title = re.sub(r'\W+', '', sheet_name)[:30]
         if header:
             for i, header_cell in enumerate(header):
-                cell = ws.cell(row=0, column=i)
+                cell = ws.cell(row=first_row, column=i+column_base)
                 cell.value = header_cell
-                cell.style.font.bold = True
+                if old_openpyxl:
+                    cell.style.font.bold = True
+                else:
+                    # https://bitbucket.org/ericgazoni/openpyxl/issue/321/unable-to-set-cell-styles-in-version-202
+                    # Immutable styles. U mad?
+                    cell.style = cell.style.copy(
+                        font=cell.style.font.copy(bold=True)
+                    )
                 if widths:
                     ws.column_dimensions[get_column_letter(i+1)].width = widths[i]
 
         for row in data:
+            row = map(text_type, row) # Otherwise openpyxl gets a type error
             try:
                 ws.append(row)
             except ValueError as e:
@@ -48,7 +67,7 @@ class DataExportMixin(object):
         """ Take a workbook and return a xlsx file response """
         if not title.endswith('.xlsx'):
             title += '.xlsx'
-        myfile = StringIO.StringIO()
+        myfile = BytesIO()
         myfile.write(save_virtual_workbook(wb))
         response = HttpResponse(
             myfile.getvalue(),
@@ -66,6 +85,8 @@ class DataExportMixin(object):
         if isinstance(data, dict):
             i = 0
             for sheet_name, sheet_data in data.items():
+                if i > 0:
+                    wb.create_sheet()
                 ws = wb.worksheets[i]
                 self.build_sheet(sheet_data, ws, sheet_name=sheet_name, header=header)
                 i += 1
@@ -84,7 +105,7 @@ class DataExportMixin(object):
         wb = self.list_to_workbook(data, title, header, widths)
         if not title.endswith('.xlsx'):
             title += '.xlsx'
-        myfile = StringIO.StringIO()
+        myfile = BytesIO()
         myfile.write(save_virtual_workbook(wb))
         return myfile
 
@@ -202,7 +223,7 @@ class DataExportMixin(object):
             # for adding properties to report rows
             display_field_paths.insert(0, 'pk')
             m2m_relations = []
-            for position, property_path in property_list.iteritems():
+            for position, property_path in property_list.items():
                 property_root = property_path.split('__')[0]
                 root_class = model_class
                 property_root_class = getattr(root_class, property_root)
@@ -258,7 +279,7 @@ class DataExportMixin(object):
                         for i, field in enumerate(display_field_paths[1:]):
                             if field in display_totals.keys():
                                 increment_total(field, display_totals, row[i])
-                        for position, display_property in property_list.iteritems():
+                        for position, display_property in property_list.items():
                             if not obj:
                                 obj = model_class.objects.get(pk=row.pop(0))
                             relations = display_property.split('__')
@@ -278,7 +299,7 @@ class DataExportMixin(object):
                                     val = None
                             values_and_properties_list[-1].insert(position, val)
                             increment_total(display_property, display_totals, val)
-                        for position, display_custom in custom_list.iteritems():
+                        for position, display_custom in custom_list.items():
                             if not obj:
                                 obj = model_class.objects.get(pk=row.pop(0))
                             val = obj.get_custom_value(display_custom)
@@ -331,9 +352,9 @@ class DataExportMixin(object):
                     if field in display_totals.keys():
                         increment_total(field, display_totals, row[i])
             row = list(row)
-            for position, choice_list in choice_lists.iteritems():
+            for position, choice_list in choice_lists.items():
                 row[position-1] = unicode(choice_list[row[position-1]])
-            for position, display_format in display_formats.iteritems():
+            for position, display_format in display_formats.items():
                 # convert value to be formatted into Decimal in order to apply
                 # numeric formats
                 try:
